<?php

class SilkPayPayModuleFrontController extends ModuleFrontController
{
    public $auth = true;

    public function init()
    {
        parent::init();
        if (Tools::getValue('ajax')) {
            switch (Tools::getValue('action')) {
                case 'getPaymentOrder':
                    $this->getPaymentOrder();
                    break;
                case 'getMerchant':
                    $this->getMerchant();
                    break;
            }
            die();
        }
    }

    public function initContent()
    {
        if (!Tools::getIsset('order')) {
            Tools::redirect('index.php?controller=order&step=1');
        }
        $id_order = Tools::getValue('order');
        $order = new Order($id_order);
        if ($order->hasBeenPaid()) {
            //如果已经支付过了
            Tools::redirect($this->context->link->getPageLink('history'));
        }

        parent::initContent();
        $order_presenter = new \PrestaShop\PrestaShop\Adapter\Presenter\Order\OrderPresenter();
        $is_h5 = $this->context->isMobile() && (strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') == false) && Configuration::get('SILKPAY_WECHAT_ENABLED_H5');
        if ($is_h5) {
            try {
                $response = $this->getJsapiPay($order);
                $this->context->smarty->assign($response);
            } catch (Exception $e) {
                $this->context->smarty->assign([
                    'is_h5' => true,
                    'has_error' => true,
                    'message' => $e->getMessage()
                ]);
            }
        } else {
            $this->context->smarty->assign([
                'order' => $order_presenter->present($order),
                'type' => Tools::getValue('type'),
                'is_mobile' => $this->context->isMobile() && strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') == false,
                'id_order' => $id_order,
                'isWeChatApp' => strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger'),
                'image_path' => $this->context->shop->getBaseURL(true),
                'alipay_plus' => (bool)Configuration::get('SILKPAY_USE_ALIPAY_PLUS'),
                'is_h5' => false,
            ]);
        }
        $this->setTemplate('module:silkpay/views/templates/front/pay.tpl');
    }

    public function setMedia()
    {
        if (version_compare(_PS_VERSION_, '1.7.7.0', '<')) {
            $this->context->controller->addJquery();
        }
        $this->context->controller->registerStylesheet('silkpay_front_css', 'modules/silkpay/views/css/front.css');
        $this->context->controller->registerJavascript('silkpay_front_js', 'modules/silkpay/views/js/front.js');
        Media::addJsDef([
            'silkpay_ajax_path' => $this->context->shop->getBaseURL(true) . 'modules/silkpay/ajax.php',
            'silkpay_ajax_path_' => $this->context->link->getModuleLink($this->module->name, 'pay'),
            'silkpay_type' => Tools::getValue('type'),
            'is_h5' => $this->context->isMobile() && (strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') == false) && Configuration::get('SILKPAY_WECHAT_ENABLED_H5')
        ]);
        return parent::setMedia();
    }

    public function getPaymentOrder()
    {

        $id_order = Tools::getValue('id_order');
        $type = Tools::getValue('type');
        $order = new Order($id_order);
        $is_paid = $order->hasBeenPaid();

        if (!$is_paid) {
            $currency = new Currency(Configuration::get('SILKPAY_ACCOUNT_CURRENCY'));
            $silkpay = new SilkPayClass();
            $params = $silkpay->getRequestParams($this->context, $order, $currency);
            try {
                switch ($type) {
                    case 'alipay':
                        $result = $this->getAlipayOrder($silkpay, $params, $order);
                        break;
                    case 'wechat':
                        $result = $this->getWechatOrder($silkpay, $params, $order);
                        break;
                    case 'union':
                    case 'visa':
                        $result = $this->getVisaOrder($silkpay, $params, $order, $type);
                        break;
                }
                die(json_encode(array_merge($result, [
                    'is_mobile' => $this->context->isMobile() && strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') == false,
                    'isWeChatApp' => strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger'),
                ])));
            } catch (Exception $e) {
                die(json_encode(['hasError' => true, 'message' => $e->getMessage()]));
            }
        } else {
            die(json_encode(['hasError' => true, 'message' => $this->module->l('This order has been paid')]));
        }
        exit();
    }

    /**
     * @param $silkpay SilkPayClass
     * @param $params array
     * @param $order Order
     * @throws Exception
     */
    public function getAlipayOrder($silkpay, $params, $order)
    {
        $isMobile = $this->context->isMobile() && strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') == false;
        if ($isMobile) {
            $params['osType'] = 'ANDROID';
            $response = $silkpay->getAlipayWapPay($params);
        } else {
            $response = $silkpay->getAlipayQrcode($params);
        }
        $alipayPlus = (bool)Configuration::get('SILKPAY_USE_ALIPAY_PLUS');
        $error = false;
        //使用支付宝+
        if ($alipayPlus) {
            //返回正确
            if ($response['error_code'] === '00000') {
                $redirect_url = ($isMobile ? $response['payment_url'] : $response['redirect_url']);
                $transactionId = $isMobile ? $response['transactionid'] : $response['Transaction'];
            } else {
                throw new Exception($response['message']);
            }
        } else {
            if ($response['response_code'] === 'OK') {
                $redirect_url = $isMobile ? $response['url'] : $response['generated_code_url'];
                $transactionId = $response['transactionid'];
            } else {
                throw new Exception($response['message']);
            }
        }
        $id_silkpay = $silkpay->getObjectIdByOrderId($order->id);
        if ($id_silkpay) {
            $silkpay->updateOrder([
                'type' => 'alipay',
                'order_amount' => $order->total_paid,
                'transactionid' => $transactionId,
                'date_add' => date('Y-m-d H:i:s')
            ], $id_silkpay);
        } else {
            $silkpay->saveOrder([
                'id_cart' => $order->id,
                'type' => 'alipay',
                'order_amount' => $order->total_paid,
                'transactionid' => $transactionId,
                'date_add' => date('Y-m-d H:i:s')
            ]);
        }
        $order->payment = 'Alipay';
        $order->save();
        return [
            'generated_code_url' => $redirect_url,
            'transactionId' => $transactionId,
            'isWeChatApp' => strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger'),
            'alipay_plus' => $alipayPlus,
        ];
    }

    /**
     * @param SilkPayClass $silkpay
     * @param array $params
     * @param Order $order
     * @return array|void
     * @throws Exception
     */
    private function getWechatOrder(SilkPayClass $silkpay, array $params, Order $order)
    {
        $is_h5 = $this->context->isMobile() && (strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') == false) && Configuration::get('SILKPAY_WECHAT_ENABLED_H5');
        if ($is_h5) {
            $response = $silkpay->getWechatH5Pay($params);
        } else {
            $response = $silkpay->getWechatQrcode($params);
        }
        if ($response['response_code'] === 'OK') {
            $id_silkpay = $silkpay->getObjectIdByOrderId($order->id);
            if ($id_silkpay) {
                $silkpay->updateOrder([
                    'type' => 'wechat',
                    'order_amount' => $order->total_paid,
                    'transactionid' => $response['transactionid'],
                    'date_add' => date('Y-m-d H:i:s')
                ], $id_silkpay);
            } else {
                $silkpay->saveOrder([
                    'id_cart' => $order->id,
                    'type' => 'wechat',
                    'order_amount' => $order->total_paid,
                    'transactionid' => $response['transactionid'],
                    'date_add' => date('Y-m-d H:i:s')
                ]);
            }
            $order->payment = 'Wechat';
            $order->save();
            return [
                'generated_code_url' => $is_h5 ? $response['h5_url'] : $response['generated_code_url'],
                'transactionId' => $response['transactionid'],
                'is_h5' => $is_h5,
            ];
        } elseif ($response['response_code'] === 'KO') {
            throw new Exception($response['message']);
        }
    }

    private function getVisaOrder(SilkPayClass $silkpay, array $params, Order $order, string $type)
    {
        if ($type === 'visa') {
            $response = $silkpay->getVisaPay($params);
        } else {
            $response = $silkpay->getUnionpay($params);
        }
        if ($response) {
            if (isset($response['redirect_url']) && isset($response['Transaction'])) {
                $this->context->smarty->assign([
                    'redirect_url' => $response['redirect_url'],
                    'transactionid' => $response['Transaction'],
                ]);
                $id_silkpay = $silkpay->getObjectIdByOrderId($order->id);
                if ($id_silkpay) {
                    $silkpay->updateOrder([
                        'type' => $type,
                        'order_amount' => $order->total_paid,
                        'transactionid' => $response['Transaction'],
                        'date_add' => date('Y-m-d H:i:s')
                    ], $id_silkpay);
                } else {
                    $silkpay->saveOrder([
                        'id_cart' => $order->id,
                        'type' => $type,
                        'order_amount' => $order->total_paid,
                        'transactionid' => $response['Transaction'],
                        'date_add' => date('Y-m-d H:i:s')
                    ]);
                }
                $order->payment = ucfirst($type);
                $order->save();
                return [
                    'generated_code_url' => $response['redirect_url'],
                    'transactionId' => $response['Transaction'],
                ];
            } else {
                PrestaShopLogger::addLog(json_encode($response), 3, 400, 'Silkpay');
                throw new Exception($response['message']);
            }
        } else {
            throw new Exception('Api Error');
        }
    }

    private function getJsapiPay(Order $order)
    {
        $silkpay = new SilkPayClass();
        $auth = $silkpay->getOpenid();
        if ($auth && isset($auth['openid'])) {
            $openid = $auth['openid'];
            $currency = new Currency(Configuration::get('SILKPAY_ACCOUNT_CURRENCY'));
            $amount = sprintf('%.' . $currency->precision . 'f', Tools::convertPriceFull($order->total_paid, new Currency($order->id_currency), $currency));
            if ($amount <= 0) {
                $amount = '0.01';
            }
            $params = [
                'amount' => $amount,
                'currency' => $currency->iso_code,
                'openid' => $openid
            ];
            $response = $silkpay->getWechatJsApi($params);
            //dump($response);
            //$response['transactionid'] = '202102080952561684868483';
            //$response['payInfo'] = '{"appId":"","timeStamp":1612777977,"nonceStr":"chtp0vxhNZldrP7zQnKntHWCfYPS7MH0","package":"prepay_id=wx08175257674988f853fd5e0f65c4420000","signType":"MD5","paySign":"D4E5E7BB5EF5E9045B03C23C89755D47"}';
            if ($response && isset($response['payInfo'])) {
                $payInfo = json_decode($response['payInfo'], true);
                Media::addJsDef([
                    'appId' => $payInfo['appId'],
                    'timeStamp' => $payInfo['timeStamp'],
                    'nonceStr' => $payInfo['nonceStr'],
                    'package' => $payInfo['package'],
                    'signType' => $payInfo['signType'],
                    'paySign' => $payInfo['paySign']
                ]);
                $id_silkpay = $silkpay->getObjectIdByOrderId($order->id);
                if ($id_silkpay) {
                    $silkpay->updateOrder([
                        'type' => 'wechat',
                        'order_amount' => $order->total_paid,
                        'transactionid' => $response['transactionid'],
                        'date_add' => date('Y-m-d H:i:s')
                    ], $id_silkpay);
                } else {
                    $silkpay->saveOrder([
                        'id_cart' => $order->id,
                        'type' => 'wechat',
                        'order_amount' => $order->total_paid,
                        'transactionid' => $response['transactionid'],
                        'date_add' => date('Y-m-d H:i:s')
                    ]);
                }
                return [
                    'total_paid' => $order->total_paid,
                    'amount' => $amount,
                    'products' => $order->getProducts(),
                    'transactionId' => $response['transactionid'],
                    'symbol' => $currency->symbol,
                    'image_path' => $this->context->shop->getBaseURL(true),
                    'is_h5' => true
                ];
            }
        } else {
            throw new Exception($auth['errmsg']);
        }
    }

    private function getMerchant()
    {
        die(json_encode([
            'uid' => Configuration::get('SILKPAY_ACCOUNT_UID'),
            'sid' => Configuration::get('SILKPAY_ACCOUNT_SID'),
        ]));
    }
}