<?php

class SilkPayValidateModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        /*
       * If the module is not active anymore, no need to process anything.
       */
        if ($this->module->active == false) {
            die;
        }
        if (!($this->module instanceof Silkpay)) {
            Tools::redirect('index.php?controller=order&step=1');
            return;
        }

        $cart = $this->context->cart;
        if ($cart->nbProducts() == 0) {
            Tools::redirect($this->context->link->getPageLink('history'));
        }
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
            return;
        }

        // Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'silkpay') {
                $authorized = true;
                break;
            }
        }
        if (!$authorized) {
            die($this->trans('This payment method is not available.', [], 'Modules.Checkpayment.Shop'));
        }

        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
            return;
        }
        $id_status = Configuration::get('SILKPAY_WAIT_STATUS');
        if (!$id_status) {
            $id_status = Configuration::get('PS_OS_BANKWIRE');
        }
        $method = Tools::getValue('method');
        $payment_names = [
            'wechat' => 'Wepay',
            'alipay' => 'Alipay',
            'union' => 'Bank card',
            'visa' => 'Bank card'
        ];
        $payment_name = 'Unkown';
        if (isset($payment_names[$method])) {
            $payment_name = $payment_names[$method];
        }
        if ($this->module->validateOrder(
            $cart->id,
            (int)$id_status,
            $cart->getOrderTotal(),
            $payment_name,
            null,
            []
        )) {
            $id_order = Order::getIdByCartId($cart->id);
            /*
            if ($method === 'visa') {
                $link = $this->context->link->getModuleLink('silkpay', 'card', ['type' => 'visa', 'order' => $id_order]);
            } elseif ($method === 'union') {
                $link = $this->context->link->getModuleLink('silkpay', 'card', ['type' => 'union', 'order' => $id_order]);
            } else {
                $link = $this->context->link->getModuleLink('silkpay', $method, ['order' => $id_order]);
            }
            */
            $link = $this->context->link->getModuleLink($this->module->name, 'pay', ['order' => $id_order,'type'=>$method]);
            Tools::redirect($link);
        }
        parent::initContent();
        $this->context->smarty->assign([
            'image_path' => $this->context->shop->getBaseURL(true),
        ]);
        $this->setTemplate('module:silkpay/views/templates/front/validate.tpl');
    }
}