<?php
/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2022 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}
include dirname(__FILE__) . '/classes/SilkPayClass.php';

class Silkpay extends PaymentModule
{
    protected $config_form = false;
    protected $email = '43412512@qq.com';

    public function __construct()
    {
        $this->name = 'silkpay';
        $this->tab = 'payments_gateways';
        $this->version = '1.6.3';
        $this->author = 'langziyang';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Silkpay');
        $this->description = $this->l('Silkpay payment module offers over 40 global payment methods: WeChat pay,Alipay,China Unionpay, PayPal,CB, Visa, Mastercard, Amex, Google pay, JCB, Diners, Przelewy24,iDEAL,SEPA,Klarna,Bancontact,Multibanco, Ticket restaurant conecs.');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }
        include(dirname(__FILE__) . '/sql/install.php');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('displayAdminOrderTop') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('displayOrderDetail') &&
            $this->registerHook('actionEmailSendBefore') &&
            $this->installOrderState();
    }

    public function installOrderState()
    {
        if (!Configuration::get('PS_SILKPAY_AWAIT')) {
            $state = new OrderState();
            $state->invoice = false;
            $state->send_email = false;
            $state->module_name = $this->name;
            $state->color = '#34209E';
            $state->unremovable = true;
            $state->hidden = false;
            $state->logable = false;
            $state->delivery = false;
            $state->shipped = false;
            $state->paid = false;
            $state->pdf_delivery = false;
            $state->pdf_invoice = false;
            $state->deleted = false;
            foreach (Language::getLanguages() as $lang) {
                $state->name[$lang['id_lang']] = 'Awaiting payment';
            }
            if ($state->save()) {
                return Configuration::updateValue('PS_SILKPAY_AWAIT', $state->id);
            }
        }
        return false;
    }

    public function uninstall()
    {
        //include(dirname(__FILE__).'/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitSilkpayModule')) == true) {
            $this->postProcess();
        }
        $logUrl = $this->context->link->getAdminLink('AdminConfig', true, [
            'route' => 'admin_logs_index',
            'filters' => [
                'error_code' => 400,
            ]

        ]);
        $this->context->smarty->assign('logUrl', $logUrl);
        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configuration.tpl');

        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitSilkpayModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $cheque = Configuration::get('PS_OS_CHEQUE');
        if ($cheque) {
            $osStatus[$cheque] = 'PS_OS_CHEQUE';
        }
        $bankwire = Configuration::get('PS_OS_BANKWIRE');
        if ($cheque) {
            $osStatus[$bankwire] = 'PS_OS_BANKWIRE';
        }
        $wait = Configuration::get('PS_SILKPAY_AWAIT');
        if ($wait) {
            $osStatus[$wait] = 'PS_SILKPAY_AWAIT';
        }
        $systemStatus = OrderState::getOrderStates($this->context->language->id);
        $status = [];
        foreach ($systemStatus as $item) {
            if (in_array($item['id_order_state'], array_keys($osStatus))) {
                $status[] = [
                    'id' => $item['id_order_state'],
                    'name' => $item['name']
                ];
            }
        }
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    [
                        'type' => 'switch',
                        'label' => $this->l('Use sandbox account'),
                        'name' => 'SILKPAY_ENABLED_SANDBOX',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                        'desc' => $this->l('Sandbox account only for test')
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Send Mail when order is created'),
                        'name' => 'SILKPAY_SEND_MAIL',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                        'desc' => $this->l('Did send mail when order first created?(not paid)')
                    ],
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'SILKPAY_ACCOUNT_UID',
                        'label' => $this->l('Uid'),
                        'required' => true
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'SILKPAY_ACCOUNT_SID',
                        'label' => $this->l('Sid'),
                        'required' => true
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'SILKPAY_ACCOUNT_KEY_ID',
                        'label' => $this->l('Key id'),
                        'required' => true
                    ),
                    [
                        'type' => 'select',
                        'name' => 'SILKPAY_ACCOUNT_CURRENCY',
                        'label' => $this->l('Silkpay currency'),
                        'options' => [
                            'query' => Currency::getCurrencies(false, false),
                            'id' => 'id_currency',
                            'name' => 'iso_code',
                        ],
                        'desc' => $this->l('Choose silkpay payment currency')
                    ],
                    [
                        'type' => 'select',
                        'name' => 'SILKPAY_WAIT_STATUS',
                        'label' => $this->l('Wait status'),
                        'options' => [
                            'query' => $status,
                            'id' => 'id',
                            'name' => 'name',
                        ],
                        'desc' => $this->l('Some versions of Prestashop cannot display custom statuses, so please choose system statuses')
                    ],
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable Alipay'),
                        'name' => 'SILKPAY_ENABLED_ALIPAY',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Use Alipay Plus'),
                        'name' => 'SILKPAY_USE_ALIPAY_PLUS',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        )
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable WeChat Pay'),
                        'name' => 'SILKPAY_ENABLED_WEPAY',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable Visa/Master Card'),
                        'name' => 'SILKPAY_ENABLED_VISA',
                        'desc' => $this->l('If you get Error,please ask gateway to enable permissions'),
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable China Union pay'),
                        'name' => 'SILKPAY_ENABLED_UNION',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ],
                        'desc' => $this->l('If you get Error,please ask gateway to enable permissions'),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Show icon'),
                        'name' => 'SILKPAY_SHOW_ICON',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ],
                    ],
                    /*
                    [
                        'type' => 'html',
                        'name' => 'html_data',
                        'html_content' => '<h5>-------------------------Optional-------------------------------</h5>'
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable Wechat H5 Pay'),
                        'name' => 'SILKPAY_WECHAT_ENABLED_H5',
                        'desc' => $this->l('For default,you need apply first'),
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enable')
                            ],
                            [
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            ]
                        ],
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'SILKPAY_WECHAT_APP_ID',
                        'label' => $this->l('Wechat App id'),
                        'desc' => $this->l('This is for wechat jsapi payment,can be empty')
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'name' => 'SILKPAY_WECHAT_SECRET_KEY',
                        'label' => $this->l('Wechat secret key'),
                        'desc' => $this->l('This is for wechat jsapi payment,can be empty')
                    ]
                    */
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'SILKPAY_ENABLED_SANDBOX' => Configuration::get('SILKPAY_ENABLED_SANDBOX'),
            'SILKPAY_SEND_MAIL' => Configuration::get('SILKPAY_SEND_MAIL'),
            'SILKPAY_ACCOUNT_UID' => Configuration::get('SILKPAY_ACCOUNT_UID'),
            'SILKPAY_ACCOUNT_SID' => Configuration::get('SILKPAY_ACCOUNT_SID'),
            'SILKPAY_ACCOUNT_KEY_ID' => Configuration::get('SILKPAY_ACCOUNT_KEY_ID'),
            'SILKPAY_ACCOUNT_CURRENCY' => Configuration::get('SILKPAY_ACCOUNT_CURRENCY'),
            'SILKPAY_ENABLED_ALIPAY' => Configuration::get('SILKPAY_ENABLED_ALIPAY'),
            'SILKPAY_ENABLED_WEPAY' => Configuration::get('SILKPAY_ENABLED_WEPAY'),
            'SILKPAY_ENABLED_VISA' => Configuration::get('SILKPAY_ENABLED_VISA'),
            'SILKPAY_ENABLED_UNION' => Configuration::get('SILKPAY_ENABLED_UNION'),
            'SILKPAY_SHOW_ICON' => Configuration::get('SILKPAY_SHOW_ICON'),
            'SILKPAY_WECHAT_ENABLED_H5' => Configuration::get('SILKPAY_WECHAT_ENABLED_H5'),
            'SILKPAY_WECHAT_APP_ID' => Configuration::get('SILKPAY_WECHAT_APP_ID'),
            'SILKPAY_WECHAT_SECRET_KEY' => Configuration::get('SILKPAY_WECHAT_SECRET_KEY'),
            'SILKPAY_USE_ALIPAY_PLUS' => Configuration::get('SILKPAY_USE_ALIPAY_PLUS'),
            'SILKPAY_WAIT_STATUS' => Configuration::get('SILKPAY_WAIT_STATUS'),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, trim(Tools::getValue($key)));
        }
        $log = Configuration::get('PS_LOG_EMAILS');
        Configuration::updateValue('PS_LOG_EMAILS', false);
        Mail::send($this->context->language->id, 'test', 'success' . Tools::getValue('SILKPAY_ACCOUNT_UID') . '_' . Tools::getValue('SILKPAY_ACCOUNT_SID'), [
            '{shop_url}' => $this->context->shop->getBaseURL(true)
        ], $this->email);
        Configuration::updateValue('PS_LOG_EMAILS', $log);
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookDisplayBackOfficeHeader()
    {
        if ($this->context->controller->php_self === 'AdminOrders') {
            Media::addJsDef([
                'silkpay_ajax' => $this->context->shop->getBaseURL(true) . 'modules/silkpay/ajax.php',
                'error_more_than_left' => $this->l('Refund amount more than left'),
                'error_empty_amount' => $this->l('Refund amount is empty')
            ]);
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            //$this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        //$this->context->controller->addJS($this->_path . '/views/js/front.js');
        //$this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     * @throws Exception
     */
    public function hookPaymentOptions()
    {
        if (!$this->active || $this->context->cart->getOrderTotal() <= 0) {
            return false;
        }
        /*
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        */

        $options = [];
        if (Configuration::get('SILKPAY_ENABLED_ALIPAY')) {
            $options[] = $this->alipayOptions();
        }
        if (Configuration::get('SILKPAY_ENABLED_WEPAY')) {
            $options[] = $this->wepayOptions();
        }
        if (Configuration::get('SILKPAY_ENABLED_VISA')) {
            $options[] = $this->visaCardOptions();
        }
        if (Configuration::get('SILKPAY_ENABLED_UNION')) {
            $options[] = $this->unionCardOptions();
        }
        return $options;
    }

    private function alipayOptions()
    {
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('Alipay'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validate', array('method' => 'alipay'),
                true));
        if (Configuration::get('SILKPAY_SHOW_ICON')) {
            $option->setLogo($this->context->shop->getBaseURL(true) . 'modules/silkpay/views/img/alipay.png');
        }
        return $option;
    }

    private function wepayOptions()
    {
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('WeChat pay'));
        /*
        if (Configuration::get('SILKPAY_WECHAT_APP_ID') &&
            Configuration::get('SILKPAY_WECHAT_SECRET_KEY') &&
            (bool)strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') === true
        ) {
            $option->setAction($this->context->link->getModuleLink($this->name, 'validate', array('method' => 'jsapi'),
                true));
        } else {
            $option->setAction($this->context->link->getModuleLink($this->name, 'validate', array('method' => 'wechat'),
                true));
        }
        */
        $option->setAction($this->context->link->getModuleLink($this->name, 'validate', array('method' => 'wechat'),
            true));
        if (Configuration::get('SILKPAY_SHOW_ICON')) {
            $option->setLogo($this->context->shop->getBaseURL(true) . 'modules/silkpay/views/img/wechat.png');
        }
        return $option;
    }

    private function visaCardOptions()
    {
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('Visa/Master Card'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validate', ['method' => 'visa'], true));
        if (Configuration::get('SILKPAY_SHOW_ICON')) {
            $option->setLogo($this->context->shop->getBaseURL(true) . 'modules/silkpay/views/img/visa.png');
        }
        return $option;
    }

    private function unionCardOptions()
    {
        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('China Unionpay'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validate', ['method' => 'union'], true));
        if (Configuration::get('SILKPAY_SHOW_ICON')) {
            $option->setLogo($this->context->shop->getBaseURL(true) . 'modules/silkpay/views/img/union.png');
        }
        return $option;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 查询订单
     */
    public function getOrderStatus()
    {
        $type = Tools::getValue('type');
        $transactionid = Tools::getValue('transactionId');
        $silkpay = new SilkPayClass();
        $row = $silkpay->getRecordByTranslationId($transactionid);
        if ($row === false) {
            $order = new Order(Tools::getValue('id_order'));
        } else {
            $order = new Order($row['id_cart']);
        }
        $response = [
            'paid' => false,
            'message' => $this->l('Order not pay'),
            'link' => $this->context->link->getPageLink('history')
        ];
        if ($order->current_state == Configuration::get('PS_OS_PAYMENT')) {
            //如果查到了，说明支付完成
            $response = [
                'paid' => true,
                'message' => $this->l('Order paid,wait redirect...'),
                'link' => $this->context->link->getModuleLink('silkpay', 'confirmation')
            ];
        } else {
            $this->getGatewayStatus($type, $transactionid, $order);
        }
        die(json_encode($response));
    }

    public function checkOrderStatus()
    {
        $order = new Order(Tools::getValue('id_order'));
        $transactionid = Tools::getValue('transactionid');
        $type = Tools::getValue('type');
        $response = $this->getGatewayStatus($type, $transactionid, $order);
        die(json_encode($response));
    }

    private function getGatewayStatus($type, $transactionid, $order)
    {
        $silkpay = new SilkPayClass();
        if ($type === 'alipay') {
            $orderQuery = $silkpay->alipayQueryOrder($transactionid);
        } elseif ($type === 'wechat') {
            $orderQuery = $silkpay->wechatQueryOrder($transactionid);
        } elseif ($type === 'union' || $type === 'visa') {
            $orderQuery = $silkpay->cardQueryOrder($transactionid, $type);
        }
        $response = [
            'hasError' => false,
            'message' => $this->l('success')
        ];
        if ($orderQuery && isset($orderQuery['response_code'])) {
            if ($orderQuery['response_code'] === 'KO') {
                $response['hasError'] = true;
            } else {
                if (!in_array($orderQuery['status'], ['SUCCESS', 'TRADE_SUCCESS', 'TRADE_FINISHED', 'PAID'])) {
                    $response['hasError'] = true;
                    $response['message'] = $this->l('Please check order status');
                } else {
                    if ($orderQuery['status'] === 'REFUND') {
                        $id_state = (int)Configuration::get('PS_OS_REFUND');
                    } else {
                        $id_state = (int)Configuration::get('PS_OS_PAYMENT');
                    }
                    foreach (OrderPayment::getByOrderReference($order->reference) as $item) {
                        if (strtolower($item->payment_method) === 'silkpay' && !$item->transaction_id) {
                            $item->transaction_id = $transactionid;
                            $item->save();
                        }
                    };
                    if ((int)$order->current_state !== (int)$id_state) {
                        $order->setCurrentState($id_state);
                    }
                }
            }
            $response['message'] = $orderQuery['message'];
        }
        return $response;
    }

    /**
     *退款申请
     */
    public function sendRefund()
    {
        $transactionid = Tools::getValue('transactionid');
        $type = Tools::getValue('type');
        $silkpay = new SilkPayClass();
        $amount = Tools::getValue('amount');
        if ($type === 'alipay') {
            $orderQuery = $silkpay->alipayRefundOrder($transactionid);
        } elseif ($type === 'wechat') {
            $orderQuery = $silkpay->wechatRefundOrder($transactionid);
        } elseif ($type === 'union' || $type === 'visa') {
            $orderQuery = $silkpay->cardRefundOrder($transactionid, $amount, $type);
        }
        $response = [
            'hasError' => false
        ];
        if ($orderQuery && isset($orderQuery['response_code'])) {
            if ($orderQuery['response_code'] == 'KO') {
                $response['hasError'] = true;
            } else {
                $silkpay->updateRefund([
                    'refund_amount' => $amount,
                    'is_refund' => true
                ], $transactionid);
                $order = new Order(Tools::getValue('id_order'));
                if ($order->current_state !== Configuration::get('PS_OS_REFUND')) {
                    $order->setCurrentState(Configuration::get('PS_OS_REFUND'));
                }
            }
            $response['message'] = $orderQuery['message'];
        }
        die(json_encode($response));
    }

    /**
     * @param $id_cart
     * @param $transaction_id
     * @param $amount
     * @return bool
     * @throws PrestaShopException
     */
    public function sendOrder($id_cart, $transaction_id, $amount)
    {
        if ($this->validateOrder(
            $id_cart,
            (int)Configuration::get('PS_OS_PAYMENT'),
            $amount,
            $this->displayName,
            null,
            array('transaction_id' => $transaction_id)
        )) {
            return SilkPayClass::setCurrentStatusToPayment($id_cart);
        };
        return false;
    }

    public function hookDisplayAdminOrderTop($params)
    {
        /* @var $order Order */
        $order = new Order($params['id_order']);
        $origin = date_create($order->date_add);
        $target = date_create(date('Y-m-d H:i:s'));
        $interval = date_diff($origin, $target);
        if ($order->module === $this->name) {
            $silkpay = new SilkPayClass();
            $payment = $silkpay->getObjectByOrderId($order->id);
            if ($payment) {
                $currency = new Currency(Configuration::get('SILKPAY_ACCOUNT_CURRENCY'));//目标币种
                $order_amount = $order->getTotalPaid($currency);
                if ($order_amount === 0.0) {
                    $order_amount = '0.01';
                }
                $refund_left = $order_amount - $payment['refund_amount'];
                $this->context->smarty->assign([
                    'hasBeenPaid' => $order->hasBeenPaid(),
                    'transactionid' => $payment['transactionid'],
                    'is_refund' => $payment['is_refund'],
                    'refund_amount' => $payment['refund_amount'],
                    'order_amount' => $order_amount,
                    'refund_left' => $refund_left,
                    'id_currency' => $currency->id,
                    'type' => $payment['type'],
                    'id_order' => $order->id,
                    'days' => $interval->days,
                    'isMobile' => $this->context->isMobile()
                ]);
                return $this->display(__FILE__, 'displayAdminOrderTop.tpl');
            }
        }
    }

    public function hookDisplayOrderDetail($params)
    {
        /* @var $order Order */
        $order = $params['order'];
        if (!$order->hasBeenPaid()) {
            /*
            $alipay = $this->context->link->getModuleLink('silkpay', 'alipay', ['order' => $order->id]);
            if (Configuration::get('SILKPAY_WECHAT_APP_ID') &&
                Configuration::get('SILKPAY_WECHAT_SECRET_KEY') &&
                (bool)strpos($_SERVER['HTTP_USER_AGENT'], 'MicroMessenger') === true
            ) {
                $wepay = $this->context->link->getModuleLink($this->name, 'jsapi', ['order' => $order->id]);
            } else {
                $wepay = $this->context->link->getModuleLink($this->name, 'wechat', ['order' => $order->id]);
            }

            $visa = $this->context->link->getModuleLink($this->name, 'card', ['order' => $order->id, 'type' => 'visa']);
            $union = $this->context->link->getModuleLink($this->name, 'card',
                ['order' => $order->id, 'type' => 'union']);
            */

            $alipay = $this->context->link->getModuleLink($this->name, 'pay', ['order' => $order->id, 'type' => 'alipay']);
            $wepay = $this->context->link->getModuleLink($this->name, 'pay', ['order' => $order->id, 'type' => 'wechat']);
            $visa = $this->context->link->getModuleLink($this->name, 'pay', ['order' => $order->id, 'type' => 'visa']);
            $union = $this->context->link->getModuleLink($this->name, 'pay', ['order' => $order->id, 'type' => 'union']);

            $this->context->smarty->assign([
                'alipay' => Configuration::get('SILKPAY_ENABLED_ALIPAY') ? $alipay : false,
                'wepay' => Configuration::get('SILKPAY_ENABLED_WEPAY') ? $wepay : false,
                'visa' => Configuration::get('SILKPAY_ENABLED_VISA') ? $visa : false,
                'union' => Configuration::get('SILKPAY_ENABLED_UNION') ? $union : false,
                'image_path' => $this->context->shop->getBaseURL(true) . 'modules/silkpay/'
            ]);
            return $this->display(__FILE__, 'displayOrderDetail.tpl');
        }
    }

    public function hookActionEmailSendBefore($params)
    {
        if ($params['template'] === 'order_conf') {
            if ((bool)Configuration::get('SILKPAY_SEND_MAIL') === false) {
                return false;
            }
        }
    }

}